﻿using SinMaiLauncher.ChildProcessHolder;

namespace SinMaiLauncher.Models;

internal class SettingReader
{
    public readonly string BaseDir = Environment.CurrentDirectory;
    public readonly string InfraDir;

    public IProcessStartInfo Maria;
    public IProcessStartInfo AquaDx;
    public IProcessStartInfo Injector;
    public IProcessStartInfo SinMai;

    public string AimeFleSinMaiDir { get; set; }
    public string AimeFleSinMaiPath { get; set; }

    public SinMaiLauncherSettingModel Settings { get; }

    public SettingReader(SinMaiLauncherSettingModel settings)
    {
        Settings = settings;

        InfraDir = Path.GetFullPath(Path.Combine(BaseDir, settings.Infra.InfraPath));

        var mariaDir = Path.GetFullPath(Path.Combine(InfraDir, settings.Infra.MariaDb.WorkingDir));
        var mariaExe = Path.GetFullPath(Path.Combine(mariaDir, settings.Infra.MariaDb.MySqlServerExe));
        var mariaArg = settings.Infra.MariaDb.MySqlServerArgs;
        Maria = new ProcessStartInfoImpl(mariaDir, mariaExe, mariaArg);

        var aquaDxDir = Path.GetFullPath(Path.Combine(InfraDir, settings.Infra.AquaDx.WorkingDir));
        var aquaDxJavaExe = Path.GetFullPath(Path.Combine(aquaDxDir, settings.Infra.AquaDx.JavaPath));
        var aquaDxJavaArg = settings.Infra.AquaDx.JavaArgs;
        AquaDx = new ProcessStartInfoImpl(aquaDxDir, aquaDxJavaExe, aquaDxJavaArg);

        var sinMaiDir = Path.GetFullPath(Path.Combine(InfraDir, settings.SinMai.WorkingDir));
        var injectorExe = Path.GetFullPath(Path.Combine(sinMaiDir, settings.SinMai.InjectExe));
        var injectorArg = settings.SinMai.InjectArgs;
        Injector = new ProcessStartInfoImpl(sinMaiDir, injectorExe, injectorArg);

        var sinMaiExe = Path.GetFullPath(Path.Combine(sinMaiDir, settings.SinMai.SinMaiExe));
        var sinMaiArg = settings.SinMai.SinMaiArgs;
        SinMai = new ProcessStartInfoImpl(sinMaiDir, sinMaiExe, sinMaiArg);

        AimeFleSinMaiDir = Path.GetFullPath(Path.Combine(sinMaiDir, settings.SinMai.AimeFileDir));
        AimeFleSinMaiPath = Path.GetFullPath(Path.Combine(AimeFleSinMaiDir, settings.SinMai.AimeFileName));

    }

    private class ProcessStartInfoImpl(string workingDir, string exe, string arg) : IProcessStartInfo
    {
        public string WorkingDir { get; } = workingDir;
        public string Exe { get; } = exe;
        public string Arg { get; } = arg;
    }
}